@@ -1,6 +1,9 @@
 # -*- coding: utf-8 -*-
 
 from imio.helpers.interfaces import IContainerOfUnindexedElementsMarker
+from imio.helpers.workflow import do_transitions as transitions  # noqa backward import compatibility
+from imio.helpers.workflow import get_state_infos  # noqa backward import compatibility
+from imio.helpers.workflow import get_transitions  # noqa backward import compatibility
 from persistent.list import PersistentList
 from plone import api
 from plone.api.content import _parse_object_provides_query
@@ -12,7 +15,6 @@ from plone.namedfile.file import NamedBlobFile
 from plone.namedfile.file import NamedBlobImage
 from plone.registry.interfaces import IRegistry
 from Products.CMFCore.interfaces import IPropertiesTool
-from Products.CMFCore.WorkflowCore import WorkflowException
 from Products.CMFPlone.utils import base_hasattr
 from Products.CMFPlone.utils import getFSVersionTuple
 from Products.CMFPlone.utils import safe_unicode
@@ -20,7 +22,6 @@ from zc.relation.interfaces import ICatalog
 from zope.annotation import IAnnotations
 from zope.component import getUtility
 from zope.component import queryUtility
-from zope.i18n import translate
 from zope.interface.interfaces import IMethod
 from zope.intid.interfaces import IIntIds
 from zope.schema._field import Bool
@@ -83,21 +84,6 @@ def get_object(parent='', oid='', title='', ptype='', obj_path=''):
     return None
 
 
-def transitions(obj, transitions, warn=False):
-    """
-        Apply multiple transitions on obj
-    """
-    workflowTool = api.portal.get_tool("portal_workflow")
-    if not isinstance(transitions, (list, tuple)):
-        transitions = [transitions]
-    for tr in transitions:
-        try:
-            workflowTool.doActionFor(obj, tr)
-        except WorkflowException as exc:
-            if warn:
-                logger.warn("Cannot apply transition '%s' on obj '%s': '%s'" % (tr, obj, exc))
-
-
 def create_NamedBlob(filepath, typ='file'):
     """
         Return a NamedBlobFile or NamedBlobImage
@@ -474,6 +460,8 @@ def uuidToObject(uuid,
         unrestricted=unrestricted)
     if res:
         res = res[0]
+    else:
+        res = None
     return res
 
 
@@ -589,19 +577,6 @@ def get_vocab_values(context, vocab_name, attr_name='token', **kwargs):
             for term in vocab_factory(context, **kwargs)._terms]
 
 
-def get_state_infos(obj):
-    """ """
-    wfTool = api.portal.get_tool('portal_workflow')
-    review_state = wfTool.getInfoFor(obj, 'review_state')
-    wf = wfTool.getWorkflowsFor(obj)[0]
-    state = wf.states.get(review_state)
-    state_title = state and state.title or review_state
-    return {'state_name': review_state,
-            'state_title': translate(safe_unicode(state_title),
-                                     domain="plone",
-                                     context=obj.REQUEST)}
-
-
 def safe_delattr(obj, attr_name):
     """ """
     if base_hasattr(obj, attr_name):
@@ -707,10 +682,15 @@ def object_ids(context, class_names):
     return res
 
 
-def get_user_fullname(username):
+def get_user_fullname(userid, none_if_no_user=False):
     """Get fullname without using getMemberInfo that is slow slow slow...
-       We get it only from mutable_properties or authentic.
-       If no fullname, return username instead nothing."""
+    We get it only from mutable_properties or authentic.
+
+   :param userid: principal id
+   :param none_if_no_user: return None if principal is not a user or is not found
+   :return: fullname or userid if fullname is empty.
+    """
+    userid = safe_unicode(userid)
     acl_users = api.portal.get_tool('acl_users')
     storages = [acl_users.mutable_properties._storage, ]
     # if authentic is available check it first
@@ -718,22 +698,23 @@ def get_user_fullname(username):
         storages.insert(0, acl_users.authentic._useridentities_by_userid)
 
     for storage in storages:
-        data = storage.get(username, None)
+        data = storage.get(userid, None)
         if data is not None:
-            fullname = ""
+            fullname = u""
             # mutable_properties
             if hasattr(data, 'get'):
-                fullname = data.get('fullname')
+                if not data.get('isGroup'):
+                    fullname = data.get('fullname')
+                elif none_if_no_user:
+                    return None
+                else:
+                    return userid
             # authentic
             else:
                 fullname = data._identities['authentic-agents'].data['fullname']
-            return fullname or username
-    return username
-
-
-def get_transitions(obj):
-    """Return the ids of the available transitions as portal_workflow.getTransitionsFor
-       will actually return a list of dict with various infos (id, title, name, ...) of
-       the available transitions."""
-    wfTool = api.portal.get_tool('portal_workflow')
-    return [tr["id"] for tr in wfTool.getTransitionsFor(obj)]
+            break
+    else:  # we didn't find this userid
+        if none_if_no_user:
+            return None
+        return userid
+    return safe_unicode(fullname) or userid
